@@ -1,8 +1,7 @@
 #include "../../src/one_input_impl.h"
 #include "../../src/platform_detect.h"
 
-#define BOOST_UT_DISABLE_MODULE
-#include "../ut/include/boost/ut.hpp"
+#include "boost/ut.hpp"
 
 #include <type_traits>
 #include <cfloat>
